# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_ptyp.ipynb.

# %% auto 0
__all__ = ['StrBytes', 'NotStrBytes', 'StrQ', 'IntQ']

# %% ../nbs/05_ptyp.ipynb 6
import os, inspect
from abc import abstractmethod
from functools import wraps, singledispatch

# %% ../nbs/05_ptyp.ipynb 8
from types import NoneType 
from typing import (Iterable, TypeGuard, )

# %% ../nbs/05_ptyp.ipynb 10
#| export


# %% ../nbs/05_ptyp.ipynb 12
#| export


# %% ../nbs/05_ptyp.ipynb 14
from .meta import (Alias, Not, Opt)

# %% ../nbs/05_ptyp.ipynb 16
@Alias(str, bytes)
class StrBytes: ...

@Not(str, bytes)
class NotStrBytes: ...

@Opt(str)
class StrQ: ...

@Opt(int)
class IntQ: ...
